/* Check book program - version 3 */
/* See the accompanying readme.txt file for more detailed comments */

/*
 * Include system header files
 *
 * these header files contain definitions and prototypes
 * that allow you to call various system supplied routines
 * (e.g., printf, gets, atoi, atof, etc.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 * Function Prototypes
 *
 * These lines tell the compiler what type of routines we
 * will be defining and using
 */

int main(void);
void do_charges(void);
void do_checks(void);
void do_deposits(void);
void do_mac(void);
void set_balance(void);

/*
 * Global Variables
 * The following variables can be seen and used by all the routines
 * in this program
 */

float balance=0.0;

/*
 * The Main Routine
 *
 * This routine is called when the program first starts.
 * What it does is determined by you, and its actions control
 * what the program does
 */

int main()
{
    /*
     * Local Variables
     *
     * These variables can be seen and used only by this routine
     */

   int opt;
   char buf[BUFSIZ];

   /*
    * Executable Code
    *
    * This is the start of the code that actually runs
    *
    * Typically, this is where you do one-time only
    * type operations that need to be done before the
    * program starts.
    */

    puts("\nWelcome to the Super Whizbang Checking Program!");

    /*
     * Main Loop
     *
     * This loop displays a menu for the user, prompts for an option,
     * and then performs the task requested by the user.
     *
     * It will keep doing this over and over again until the user
     * asks to quit (option 9).
     */

   do
   { /* start of main loop */

    /* display the current balance */
      printf("Your balance is now %.2lf\n", balance);

      /* display a menu of choices for the user */
      puts("\nOptions are:");
      puts("1. Set balance");
      puts("2. Process checks");
      puts("3. Process deposits");
      puts("4. Process MAC transactions");
      puts("5. Process service charges");
      puts("9. Exit");
      printf("Enter option: "); /* ask for option from the user */

      /* get the user's option and process it */
      gets(buf); /* get a line from the user */
      opt = atoi(buf);  /* convert the line into its numeric value */
      switch(opt)
      { /* branch based on the user's option */
         case 1: set_balance(); break;
         case 2: do_checks(); break;
         case 3: do_deposits(); break;
         case 4: do_mac(); break;
         case 5: do_charges(); break;
         case 9: puts("Bye!"); break;
         default: puts("Invalid option - try again");
      }
   } while (opt != 9); /* end of the main loop */

   /*
    * if we get here, then the user asked to quit the program
    *
    * At this point, we should do any clean up that is needed
    * and then leave in an orderly fashion
    */

   puts("Thanks for using Super Whizbang Checking!  Come again!");
   
   return 0;    /* normal return */
}

/* the following routines handle the various menu options */

/*
 * 					**** Do Charges ****
 *
 * This routine handles service charges and other misc. charges
 * to the account.
 *
 * The routine prompts the user for necessary information,
 * displays a line representing the transaction, and updates
 * the account balance.
 *
 * When reading responses from the user, we use an input buffer
 * that is plenty big.  We then make sure that the user didn't
 * type too many characters.  If we didn't do that, we risk
 * corrupting the data and causing errors
 *
 */
void do_charges(void)
{
   char buf[BUFSIZ];                /* input buffer */
   char chg_date[9], chg_desc[21];  /* date and description */
   float chg_amt;                   /* amount of charge */

   /* get the date of the charge */
   printf("Enter date of charge (MM/DD/YY): ");
   gets(buf);
   buf[8] = '\0';   /* make sure user didn't type too much */
   strcpy(chg_date, buf);   /* copy data to correct field */

   /* now get a description of the charge */
   printf("Enter description of charge: ");
   gets(buf);
   buf[20] = '\0';
   strcpy(chg_desc, buf); /* copy data to description field */

   /* now handle the amount of the charge */
   printf("Enter amount of charge: ");
   gets(buf);
   chg_amt = atof(buf); /* convert amount to floating point */

   /* display a line showing the transaction */
   printf("%-8s %-20s %10.2f\n", chg_date, chg_desc, chg_amt);

   /* update the balance by subtracting the amount from the balance */
   balance -= chg_amt;
}

void do_checks(void)
{
   char buf[BUFSIZ], chk_date[9], chk_desc[21];
   int chk_no;
   float chk_amt;

   printf("Enter check number: ");
   gets(buf);
   chk_no = atoi(buf);  /* convert to integer */

   printf("Enter date of check: ");
   gets(buf);
   buf[8] = 0;
   strcpy(chk_date, buf);

   printf("Enter desciption of check: ");
   gets(buf);
   buf[20] = 0;
   strcpy(chk_desc, buf);

   printf("Enter amount of check: ");
   gets(buf);
   chk_amt = atof(buf);

   printf("%4d %-8s %-20s %10.2f\n", chk_no, chk_date, chk_desc, chk_amt);

   balance -= chk_amt;
}

void do_deposits(void)
{
   char buf[BUFSIZ], dep_date[8], dep_desc[20];
   float dep_amt;

   printf("Enter date of deposit: ");
   gets(buf);
   buf[8] = 0;
   strcpy(dep_date, buf);

   printf("Enter description of deposit: ");
   gets(buf);
   buf[20] = 0;
   strcpy(dep_desc, buf);

   printf("Enter amount of deposit: ");
   gets(buf);
   dep_amt = atof(buf);

   printf("%-8s %-20s %10.2f\n", dep_date, dep_desc, dep_amt);

   balance += dep_amt;
}

void do_mac(void)
{
   char buf[BUFSIZ], mac_date[9], mac_desc[21], mac_xact;
   float mac_amt;
   int sign;

   /* keep looping and processing transactions until done */
   while (1)
   {
    /* display a brief menu for the user */
      printf("Your balance is now %.2f\n", balance);
      printf("Deposit, Withdrawl, or quit? (d, w, q): ");
      gets(buf);        /* get a response from the user */
      mac_xact = buf[0]; /* get the first letter of the response */
      switch(mac_xact)
      { /* now branch to appropriate location */
        /* this next line converts lower case w to upper case W */
         case 'w': mac_xact = 'W';
         case 'W':
              sign = -1;    /* subtract amount from total*/
              break;
         case 'd': mac_xact = 'D';
         case 'D':
              sign = 1;     /* add amount to total */
              break;
         case 'q':
         case 'Q':
              return;       /* quit the loop */
         default:
                puts("Invalid transaction.  Please try again.");
                continue;
      }

      /* doing a transaction - handle the common fields */

      /* get date of transaction */
      printf("Enter date of transaction: ");
      gets(buf);
      buf[8] = 0;
      strcpy(mac_date, buf);

      /* get description of transaction */
      printf("Enter description: ");
      gets(buf);
      buf[20] = 0;
      strcpy(mac_desc, buf);

      /* get amount of transaction */
      printf("Enter amount: ");
      gets(buf);
      mac_amt = atof(buf);

      /* display line representing the transaction */
      sprintf(buf, "MAC%c", mac_xact);
      printf("%-8s %s %-20s %10.2f\n",
             mac_date, buf, mac_desc, mac_amt);

      /* update balance according to sign */
      if (sign == 1) balance += mac_amt; else balance -= mac_amt;
   }

}

/*
 * Set Balance
 *
 * This routine allows the user to set the starting balance
 * It would typically be used at the start of the program.
 * If we were able to remember what the closing balance was
 * the last time we ran, we wouldn't need this routine.
 */

void set_balance(void)
{
   char buf[BUFSIZ];

   printf("Enter new balance: "); /* ask for the new balance */
   gets(buf);           /* get a string from the user */
   balance = atof(buf); /* convert string the user typed into a number */
   /*
    * Since balance is a global variable, this change is permanent.
    *
    * If we were to define a local variable in this routine called
    * balance, then when we left this routine, the global balance
    * would be unchanged
    */
}

